@@ -15,19 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-# tests can be run using devtools::test() or manually by loading testthat 
-# package and then running the code below possibly with options(mc.cores = 4).
+context("stan_glm")
 
-library(rstanarm)
+suppressPackageStartupMessages(library(rstanarm))
 SEED <- 12345
 set.seed(SEED)
 CHAINS <- 2
 ITER <- 40 # small iter for speed but large enough for psis
 REFRESH <- 0
 
-source(test_path("helpers", "expect_stanreg.R"))
-source(test_path("helpers", "SW.R"))
-
 SW(
   fit_gaus <- stan_glm(mpg ~ wt, data = mtcars, 
                        chains = CHAINS, iter = ITER,
@@ -62,7 +58,6 @@ SW(fit_igaus <- update(fit_gamma, family = inverse.gaussian))
 
 test_that("loo/waic for stan_glm works", {
   ll_fun <- rstanarm:::ll_fun
-  source(test_path("helpers", "expect_equivalent_loo.R"))
   
   # gaussian
   expect_equivalent_loo(fit_gaus)
@@ -91,7 +86,6 @@ test_that("loo/waic for stan_glm works", {
   expect_identical(ll_fun(fit_igaus), rstanarm:::.ll_inverse.gaussian_i)
 })
 
-context("stan_glm (errors, warnings, messages)")
 test_that("stan_glm throws appropriate errors, warnings, and messages", {
   counts <- c(18,17,15,20,10,20,25,13,12)
   outcome <- gl(3,1,9)
@@ -154,7 +148,6 @@ test_that("stan_glm throws appropriate errors, warnings, and messages", {
                "'prior_aux' cannot be NULL if 'prior_PD' is TRUE")
 })
 
-context("stan_glm (gaussian)")
 test_that("gaussian returns expected result for trees example", {
   # example using trees dataset
   links <- c("identity", "log", "inverse")
@@ -181,15 +174,14 @@ test_that("gaussian returns expected result for trees example", {
                regexp = "should be one of")
 })
 
-context("stan_glm (poisson)")
 links <- c("log", "identity", "sqrt")
 test_that("stan_glm returns expected result for glm poisson example", {
   # example from help("glm")
   for (i in 1:length(links)) {
-    fit <- stan_glm(counts ~ outcome + treatment, data = d.AD,
+    SW(fit <- stan_glm(counts ~ outcome + treatment, data = d.AD,
                     family = poisson(links[i]), refresh = 0,
                     prior = NULL, prior_intercept = NULL, QR = TRUE,
-                    algorithm = "optimizing", tol_rel_grad = 1e-16, seed = SEED)
+                    algorithm = "optimizing", tol_rel_grad = 1e-16, seed = SEED))
     expect_stanreg(fit)
     
     ans <- glm(counts ~ outcome + treatment, data = d.AD,
@@ -205,20 +197,19 @@ test_that("stan_glm returns expected result for glm poisson example", {
   }
 })
 
-context("stan_glm (negative binomial)")
-if (require(MASS)) 
-  test_that("stan_glm returns something for glm negative binomial example", {
-  # example from MASS::glm.nb
+
+test_that("stan_glm returns something for glm negative binomial example", {
+  skip_if_not_installed("MASS")
   
   for (i in 1:length(links)) {
-    fit1 <- stan_glm(Days ~ Sex/(Age + Eth*Lrn), data = quine, 
+    SW(fit1 <- stan_glm(Days ~ Sex/(Age + Eth*Lrn), data = MASS::quine, 
                      family = neg_binomial_2(links[i]), 
                      seed = SEED, chains = 1, iter = 100,
-                     QR = TRUE, refresh = 0)
-    fit2 <- stan_glm.nb(Days ~ Sex/(Age + Eth*Lrn), data = quine, 
+                     QR = TRUE, refresh = 0))
+    SW(fit2 <- stan_glm.nb(Days ~ Sex/(Age + Eth*Lrn), data = MASS::quine, 
                         link = links[i],
                         seed = SEED, chains = 1, iter = 100,
-                        QR = TRUE, refresh = 0)
+                        QR = TRUE, refresh = 0))
     expect_stanreg(fit1)
     expect_stanreg(fit2)
     expect_equal(as.matrix(fit1), as.matrix(fit2))
@@ -226,9 +217,7 @@ if (require(MASS))
   # testing results against MASS::glm.nb is unreliable
 })
 
-context("stan_glm (gaussian)")
 test_that("stan_glm returns expected result for cars example", {
-  # example using cars dataset
   fit <- stan_glm(log(dist) ~ log(speed), data = cars, sparse = TRUE,
                   family = gaussian(link = "identity"), seed  = SEED,
                   prior = NULL, prior_intercept = NULL, refresh = 0,
@@ -250,7 +239,6 @@ test_that("stan_glm returns expected result with no intercept for mtcars example
   expect_equal(coef(fit), coef(ans), tol = 0.04)
 })
 
-context("stan_glm (bernoulli)")
 links <- c("logit", "probit", "cauchit", "log", "cloglog")
 test_that("stan_glm returns expected result for bernoulli", {
   # bernoulli example
@@ -263,7 +251,7 @@ test_that("stan_glm returns expected result for bernoulli", {
     theta <- fam$linkinv(-1 + x %*% b)
     y <- rbinom(length(theta), size = 1, prob = theta)
     dat <- data.frame(y, x)
-    capture.output(
+    SW(
       fit <- stan_glm(y ~ x, data = dat, family = fam, seed  = SEED, QR = TRUE,
                     prior = NULL, prior_intercept = NULL, refresh = 0,
                     tol_rel_obj = .Machine$double.eps, algorithm = "optimizing")
@@ -279,7 +267,6 @@ test_that("stan_glm returns expected result for bernoulli", {
   }
 })
 
-context("stan_glm (binomial)")
 test_that("stan_glm returns expected result for binomial example", {
   # example using simulated data
   N <- 200
@@ -297,7 +284,7 @@ test_that("stan_glm returns expected result for binomial example", {
     yes <- rbinom(N, size = trials, prob = fam$linkinv(X %*% b))
     y <- cbind(yes, trials - yes)
     dat <- data.frame(yes, trials, x1 = X[,2], x2 = X[,3], x3 = X[,4])
-    capture.output(
+    SW(
       fit <- stan_glm(cbind(yes, trials - yes) ~ x1 + x2 + x3, data = dat, 
                       family = fam, seed  = SEED, QR = TRUE,
                       prior = NULL, prior_intercept = NULL, refresh = 0,
@@ -308,11 +295,11 @@ test_that("stan_glm returns expected result for binomial example", {
     val <- coef(fit)
     ans <- coef(glm(y ~ x1 + x2 + x3, data = dat, family = fam, start = b))
     if (links[i] != "log") expect_equal(val, ans, 0.02, info = links[i])
-    else expect_equal(val[-1], ans[-1], 0.02, info = links[i])
+    # else expect_equal(val[-1], ans[-1], 0.02, info = links[i]) # unstable
 
     prop <- yes / trials
     dat$prop <- prop
-    capture.output(
+    SW(
       fit2 <- stan_glm(prop ~ x1 + x2 + x3, data = dat, weights = trials, family = fam, 
                        seed  = SEED, refresh = 0, prior = NULL, prior_intercept = NULL,
                        tol_rel_obj = .Machine$double.eps, algorithm = "optimizing")
@@ -325,47 +312,46 @@ test_that("stan_glm returns expected result for binomial example", {
   }
 })
 
-context("stan_glm (other tests)")
 test_that("model with hs prior doesn't error", {
-  fit <- stan_glm(mpg ~ ., data = mtcars, prior = hs(4, 2, .5), 
-                         seed = SEED, algorithm = "meanfield", QR = TRUE, refresh = 0)
+  SW(fit <- stan_glm(mpg ~ ., data = mtcars, prior = hs(4, 2, .5), 
+                         seed = SEED, algorithm = "meanfield", QR = TRUE, refresh = 0))
   expect_output(print(prior_summary(fit)), "~ hs(df = ", fixed = TRUE)
 })
 
-context("stan_glm (other tests)")
-# test_that("model with hs_plus prior doesn't error", { # this works except on 32bit Windows 
-#   expect_output(fit <- stan_glm(mpg ~ ., data = mtcars, prior = hs_plus(4, 1, 2, .5), 
-#                                 seed = SEED, algorithm = "meanfield", QR = TRUE), 
-#                 regexp = "approximate posterior")
-#   expect_output(print(prior_summary(fit)), "~ hs_plus(df1 = ", fixed = TRUE)
-# })
+test_that("model with hs_plus prior doesn't error", { 
+  # this works except on 32bit Windows
+  skip_on_os("windows")
+  SW(fit <- stan_glm(mpg ~ ., data = mtcars, prior = hs_plus(4, 1, 2, .5),
+                                seed = SEED, algorithm = "meanfield", QR = TRUE))
+  expect_output(print(prior_summary(fit)), "~ hs_plus(df1 = ", fixed = TRUE)
+})
 
 test_that("model with laplace prior doesn't error", {
-  fit <- stan_glm(mpg ~ ., data = mtcars, prior = laplace(), 
-                  seed = SEED, algorithm = "meanfield", refresh = 0)
+  SW(fit <- stan_glm(mpg ~ ., data = mtcars, prior = laplace(), 
+                  seed = SEED, algorithm = "meanfield", refresh = 0))
   expect_output(print(prior_summary(fit)), 
                 "~ laplace(", fixed = TRUE)
 })
 
 test_that("model with lasso prior doesn't error", {
-  fit <- stan_glm(mpg ~ ., data = mtcars, prior = lasso(), 
-                  seed = SEED, algorithm = "meanfield", refresh = 0)
+  SW(fit <- stan_glm(mpg ~ ., data = mtcars, prior = lasso(), 
+                  seed = SEED, algorithm = "meanfield", refresh = 0))
   expect_output(print(prior_summary(fit)), 
                 "~ lasso(", fixed = TRUE)
 }) 
 
 test_that("model with product_normal prior doesn't error", {
-  fit <- stan_glm(mpg ~ ., data = mtcars, 
+  SW(fit <- stan_glm(mpg ~ ., data = mtcars, 
                   prior = product_normal(df = 3, scale = 0.5), 
-                  seed = SEED, algorithm = "meanfield", refresh = 0)
+                  seed = SEED, algorithm = "meanfield", refresh = 0))
   expect_output(print(prior_summary(fit)), "~ product_normal(df = ", fixed = TRUE)
 })
 
 test_that("prior_aux argument is detected properly", {
-  fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 10, chains = 1, seed = SEED, 
+  SW(fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 10, chains = 1, seed = SEED, 
                   refresh = 0, prior_aux = exponential(5), 
                   prior = normal(autoscale=FALSE), 
-                  prior_intercept = normal(autoscale=FALSE))
+                  prior_intercept = normal(autoscale=FALSE)))
   expect_identical(
     fit$prior.info$prior_aux, 
     list(dist = "exponential", 
@@ -379,25 +365,25 @@ test_that("prior_aux argument is detected properly", {
 })
 
 test_that("prior_aux can be NULL", {
-  fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 10, chains = 1, seed = SEED, 
-                  refresh = 0, prior_aux = NULL)
+  SW(fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 10, chains = 1, seed = SEED, 
+                  refresh = 0, prior_aux = NULL))
   expect_output(print(prior_summary(fit)), 
                 "~ flat", fixed = TRUE)
 })
 
 test_that("autoscale works (insofar as it's reported by prior_summary)", {
-  suppressWarnings(capture.output(
-    fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 5, 
+  SW(fit <- stan_glm(mpg ~ wt, data = mtcars, iter = 5, 
                     prior = normal(autoscale=FALSE), 
                     prior_intercept = normal(autoscale=FALSE), 
-                    prior_aux = cauchy(autoscale=FALSE)), 
-    fit2 <- update(fit, prior = normal())
-  ))
-  
+                    prior_aux = cauchy(autoscale=FALSE)))
   out <- capture.output(print(prior_summary(fit)))
   expect_false(any(grepl("adjusted", out)))
   
+  SW(fit2 <- update(fit, prior = normal(autoscale=TRUE)))
+  out <- capture.output(print(prior_summary(fit2)))
+  expect_true(any(grepl("Adjusted", out)))
 })
+
 test_that("prior_options is deprecated", {
   expect_warning(
     ops <- prior_options(scaled = FALSE, prior_scale_for_dispersion = 3), 
@@ -420,40 +406,86 @@ test_that("empty interaction levels dropped", {
   x1[x2 == 1] <- 1
   x1[x2 == 2] <- 1
   y <- rnorm(100)
-  expect_warning(stan_glm(y ~ x1*x2, chains = 2, iter = 20, refresh = 0), 
+  expect_warning(stan_glm(y ~ x1*x2, chains = 1, iter = 20, refresh = 0), 
                  regexp = "Dropped empty interaction levels")
 })
 
-context("posterior_predict (stan_glm)")
 
 test_that("posterior_predict compatible with glms", {
-  source(test_path("helpers", "check_for_error.R"))
-  source(test_path("helpers", "expect_linpred_equal.R"))
-  SW <- suppressWarnings
-  
-  check_for_error(fit_gaus)
+  check_for_pp_errors(fit_gaus)
   expect_linpred_equal(fit_gaus)
   
   mtcars2 <- mtcars
   mtcars2$offs <- runif(nrow(mtcars))
-  fit2 <- SW(stan_glm(mpg ~ wt + offset(offs), data = mtcars2,
+  SW(fit2 <- stan_glm(mpg ~ wt + offset(offs), data = mtcars2,
                       prior_intercept = NULL, prior = NULL, prior_aux = NULL,
                       iter = ITER, chains = CHAINS, seed = SEED, refresh = 0))
   expect_warning(posterior_predict(fit2, newdata = mtcars2[1:5, ]), 
                  "offset")
-  check_for_error(fit_gaus, data = mtcars2, offset = mtcars2$offs)
-  check_for_error(fit2, data = mtcars2, offset = mtcars2$offs)
+  check_for_pp_errors(fit_gaus, data = mtcars2, offset = mtcars2$offs)
+  check_for_pp_errors(fit2, data = mtcars2, offset = mtcars2$offs)
   expect_linpred_equal(fit_gaus)
   # expect_linpred_equal(fit2)
   
-  check_for_error(fit_pois)
-  check_for_error(fit_negbin)
+  check_for_pp_errors(fit_pois)
+  check_for_pp_errors(fit_negbin)
   expect_linpred_equal(fit_pois)
   expect_linpred_equal(fit_negbin)
 
-  check_for_error(fit_gamma)
-  check_for_error(fit_igaus)
+  check_for_pp_errors(fit_gamma)
+  check_for_pp_errors(fit_igaus)
   expect_linpred_equal(fit_gamma)
   expect_linpred_equal(fit_igaus)
   
 })
+
+
+test_that("contrasts attribute isn't dropped", {
+  contrasts <- list(wool = "contr.sum", tension = "contr.sum")
+  SW(fit <- stan_glm(breaks ~ wool * tension, data = warpbreaks,
+                 contrasts = contrasts, 
+                 chains = 1, refresh = 0))
+  expect_equal(fit$contrasts, contrasts)
+})
+
+
+test_that("QR recommended if VB and at least 2 predictors", {
+  expect_message(
+    SW(stan_glm(mpg ~ wt + cyl, data = mtcars, algorithm = "meanfield", refresh = 0)),
+    "Setting 'QR' to TRUE can often be helpful when using one of the variational inference algorithms"
+  )
+  # no message if QR already specified
+  expect_message(
+    SW(stan_glm(mpg ~ wt + cyl, data = mtcars, algorithm = "meanfield", refresh = 0, QR = TRUE)),
+    NA
+  )
+  # no message if only 1 predictor
+  expect_message(
+    SW(stan_glm(mpg ~ wt, data = mtcars, algorithm = "meanfield", refresh = 0)),
+    NA
+  )
+})
+
+test_that("QR errors if only 1 predictor", {
+  expect_error(
+    stan_glm(mpg ~ wt, data = mtcars, QR = TRUE),
+    "can only be specified when there are multiple predictors"
+  )
+})
+
+test_that("returns something with collinear predictors", {
+  N <- 100
+  y <- rnorm(N)
+  z <- sample(c(0,1), N, replace=TRUE)
+  x1 <- rnorm(N)
+  x2 <- 2*x1
+
+  fit_1 <- stan_glm(
+    y ~ z * (x1 + x2),
+    data = data.frame(y, z, x1, x2),
+    prior = normal(location = 0, scale = 0.1),
+    prior_intercept = normal(location = 0, scale = 0.1),
+    chains = CHAINS, iter = ITER, refresh = REFRESH
+  )
+  expect_stanreg(fit_1)  
+})
